//#include <sys/time.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <strings.h>

#include "rtp.h"
#include "UserData.h"

unsigned int UserData::start_seconds = 0;
int UserData::first = 1;

UserData::UserData(RTPheader *hdr, int size)
{
	frames_received = 0;
	frames_lost = 0;
	frames_late = 0;
	latency = 0;
	last_ts = get_time_stamp();
	last_seq = hdr->seq;
	first_time = get_time_stamp();
	bytes = size;

	reset();
}

int UserData::frame(RTPheader *hdr, int size)
{
	unsigned int tmp;
	int ok = 1;

	//if(frames_received == 100)
	//	reset();
	frames_received++;
	bytes += size;
	if(frames_received == 1)
	{
		last_ts = get_time_stamp();
		last_seq = hdr->seq;
		first_time = get_time_stamp();
	}
	else
	{
		if(hdr->seq < last_seq)
		{
			frames_lost++;
			ok = 0;
		}
		else
		  last_seq = hdr->seq;

		tmp = get_time_stamp();
		if((tmp > last_ts + 300) && (tmp < last_ts + 1000))
			frames_late++;
		latency += tmp - last_ts - 100;
		last_ts = tmp;

		timestamps[pos] = tmp;
		sizes[pos] = size;
		pos = ++pos % WINDOW_MAX;

	}

	return ok;
}

int UserData::getPercentLost()
{
	if(frames_received == 0)
		return 0;

	return frames_lost*100/(frames_received + frames_lost);
}

int UserData::getPercentLate()
{
	if(frames_received == 0)
		return 0;

	return frames_late*100/(frames_received + frames_lost);
}

void UserData::reset()
{
	int i;
	
	frames_received = 0;
	frames_lost = 0;
	frames_late = 0;
	bytes = 0;
	latency = 0;

	pos = 0;
	for(i=0; i<WINDOW_MAX; i++)
	{
		timestamps[i] = 0;
		sizes[i] = 0;
	}
}

double UserData::getFps()
{
	unsigned int time;
	double diff;
	int packets = 0;
	int i;

	time = get_time_stamp();

	time -= WINDOW_SIZE;
	for(i=0; i<WINDOW_MAX; i++)
	{
		if(timestamps[i] > time)
		{
			packets++;
		}
	}

	return (double)((double)1000 * (double)packets / (double)WINDOW_SIZE);
	/*
	diff = time - first_time;
	if(diff == 0)
		return 0;
	return (double)1000 * (double)frames_received / diff;
*/
}

double UserData::getKbps()
{
	unsigned int time;
	double diff;
	int size = 0;
	int i;

	time = get_time_stamp();

	time -= WINDOW_SIZE;
	for(i=0; i<WINDOW_MAX; i++)
	{
		if(timestamps[i] > time)
		{
			size += sizes[i];
		}
	}

	return 8 * size / WINDOW_SIZE;
/*
	diff = time - first_time;
	if(diff == 0)
		return 0;
	return (double)8 * (double)bytes / diff;
	*/
}

double UserData::getLatency()
{
	if(frames_received != 0)
		return (double)latency/(double)frames_received;
	
	return 0;
}

unsigned int UserData::get_time_stamp()
{
  unsigned int result;
  struct timeval time;

  gettimeofday(&time);
  if(UserData::first)
    {
      UserData::start_seconds = time.tv_sec;
      UserData::first = 0;
    }

  result = (time.tv_sec - UserData::start_seconds)*1000 + (unsigned int)(time.tv_usec/1000);
  
  return result;
}
